class GenericBuilder < Builder
  ##
  # receive a list of ids and return a list of learning objects
  # useful for search pagination
  #
  def self.build(objects = [])
    build_array(objects, true, 'rid')
  end

  private

  def self.build_array(objects = [], cache = true, id = '@rid', last_modified = 'last_modified')
    array = []
    objects = [objects] if objects.class == String || objects.class == Hash
    objects.each do |object|
      next if object[id].blank?

      if cache && !object[last_modified].blank?
        o = Rails.cache.fetch(cache_key(object[id], object[last_modified]))
      end

      o = repository(object['class']).find object[id] if o.nil?
      array << o
    end
    array
  end

  def self.repository(klass)
    case klass
    when 'Attribute' then attribute_repository
    when 'Collection' then collection_repository
    when 'Complaint' then complaint_repository
    when 'Institution' then institution_repository
    when 'LearningObject' then learning_object_repository
    when 'Subject' then subject_repository
    else nil
    end
  end

end
