module OrientDb
  class CollectionRepository < Base
    include OrientDb::Methods::EdgeMethods
    include OrientDb::Methods::FinderMethods
    include RepositoriesProxy

    def save_learning_objects(collection = Collection.new)
      if collection.id.to_s.empty?
        raise NotPersistedRecordError, 'Before save learning objects to collection, you must persist using CollectionRepository::create method'
      end

      field = collection.class == Collection ? 'learning_objects' : 'bookmark'

      connection.command sprintf('update %s REMOVE %s', collection.id, field)
      connection.command sprintf("update %s ADD %s = [%s] SET last_modified='%s'", collection.id, field, collection.learning_objects.map(&:id).join(','), Time.now())
    end

    def get_number_of_collections
      get_number_of odb_class
    end

    def create(object)
      super(object)

      # create relationship
      create_edge "BelongsTo", object.id, owner_id(object.owner)

      object
    end

    ##
    # Select all collections in context
    def all(context = Collections::PublicContext.new)
      query = sprintf("select * from %s", context.from)
      query += sprintf(" WHERE privacy = '%s'", context.privacy) unless context.privacy.blank?
      build_objects connection.query(query)
    end

    def find_by_learning_object(learning_object)
      if learning_object.id.to_s.empty?
        return []
      end

      query = sprintf("select * from %s where learning_objects contains %s", odb_class, learning_object.id)
      build_objects connection.query(query)
    end

    def bookmarks(user)
      user_result = connection.query sprintf("select from User where @rid = %s", user.rid)
      BookmarksBuilder.build_from_orientdb user_result.first
    end

    # return owner of collection rid
    def owner(collection)
      collection_id = collection
      collection_id = collection.id if collection.class == Collection

      edge = get_edges("BelongsTo", "out", collection_id).first
      owner = get_by_rid(edge["in"])

      if owner['@class'] == "User"
        return user_repository.find owner['p_id']
      elsif owner['@class'] == "Institution"
        return institution_repository.find owner['@rid']
      end

      []
    end

    def owner?(collection, user)
      return false if collection.blank? || user.blank?

      if collection.owner.class == Institution
        user.institutions.each do |institution|
          return true if institution.id == collection.owner.id
        end
      else
        # if collection owner is a user
        return true if collection.owner.rid == user.rid
      end

      false
    end

    def change_privacy(collection, privacy)
      update_property(collection, 'privacy', privacy) if privacy_accepted?(privacy)
    end

    protected

    def accepted_properties
      ['privacy']
    end

    def privacy_accepted?(privacy)
      ['private', 'public'].include? privacy
    end

    def owner_id(owner)
      return owner if owner.class != User && owner.class != Institution
      (owner.class == User) ? owner.rid : owner.id
    end

  end
end
